@@ -60,20 +60,6 @@
 		 */
 		imgPath: mw.config.get( 'wgExtensionAssetsPath' ) + '/WikiEditor/modules/images/',
 
-		/**
-		 * Checks if the client supports WikiEditor.
-		 *
-		 * Since 1.31 this check is deprecated and can be skipped as all browsers
-		 * which are served JS by MediaWiki support WikiEditor.
-		 *
-		 * @deprecated since 1.31
-		 * @return {boolean}
-		 */
-		isSupported: function () {
-			mw.log.warn( '$.wikiEditor.isSupported is deprecated.' );
-			return true;
-		},
-
 		/**
 		 * Checks if a module has a specific requirement
 		 *
@@ -96,6 +82,10 @@
 		/**
 		 * Provides a way to extract messages from objects. Wraps a mw.message( ... ).text() call.
 		 *
+		 * FIXME: This is a security nightmare. Only use is for the help toolbar panel. Inline the
+		 *        special need instead?
+		 * FIXME: Also, this is ludicrously complex. Just use mw.message().text() directly.
+		 *
 		 * @param {Object} object Object to extract messages from
 		 * @param {string} property String of name of property which contains the message. This should be the base name of the
 		 * property, which means that in the case of the object { this: 'that', fooMsg: 'bar' }, passing property as 'this'
@@ -128,6 +118,43 @@
 			}
 		},
 
+		/**
+		 * Provides a way to extract messages from objects. Wraps a mw.message( ... ).escaped() call.
+		 *
+		 * FIXME: This is ludicrously complex. Just use mw.message().escaped() directly.
+		 *
+		 * @param {Object} object Object to extract messages from
+		 * @param {string} property String of name of property which contains the message. This should be the base name of the
+		 * property, which means that in the case of the object { this: 'that', fooMsg: 'bar' }, passing property as 'this'
+		 * would return the raw text 'that', while passing property as 'foo' would return the internationalized message
+		 * with the key 'bar'. This is then escaped.
+		 * @return {string}
+		 */
+		autoSafeMsg: function ( object, property ) {
+			var i, p;
+			// Accept array of possible properties, of which the first one found will be used
+			if ( typeof property === 'object' ) {
+				for ( i in property ) {
+					if ( property[ i ] in object || property[ i ] + 'Msg' in object ) {
+						property = property[ i ];
+						break;
+					}
+				}
+			}
+			if ( property in object ) {
+				return object[ property ];
+			} else if ( property + 'Msg' in object ) {
+				p = object[ property + 'Msg' ];
+				if ( Array.isArray( p ) && p.length >= 2 ) {
+					return mw.message.apply( mw.message, p ).escaped();
+				} else {
+					return mw.message( p ).escaped();
+				}
+			} else {
+				return '';
+			}
+		},
+
 		/**
 		 * Provides a way to extract a property of an object in a certain language, falling back on the property keyed as
 		 * 'default' or 'default-rtl'. If such key doesn't exist, the object itself is considered the actual value, which
