@@ -43,7 +43,7 @@ abstract class AbstractContent implements Content {
 	protected $model_id;
 
 	/**
-	 * @param string $modelId
+	 * @param string|null $modelId
 	 *
 	 * @since 1.21
 	 */
@@ -145,7 +145,7 @@ abstract class AbstractContent implements Content {
 	/**
 	 * @since 1.21
 	 *
-	 * @param string $format
+	 * @param string|null $format
 	 *
 	 * @return string
 	 *
@@ -182,7 +182,7 @@ abstract class AbstractContent implements Content {
 	/**
 	 * @since 1.21
 	 *
-	 * @param Content $that
+	 * @param Content|null $that
 	 *
 	 * @return bool
 	 *
@@ -219,9 +219,9 @@ abstract class AbstractContent implements Content {
 	 * @since 1.21
 	 *
 	 * @param Title $title
-	 * @param Content $old
+	 * @param Content|null $old
 	 * @param bool $recursive
-	 * @param ParserOutput $parserOutput
+	 * @param ParserOutput|null $parserOutput
 	 *
 	 * @return DataUpdate[]
 	 *
@@ -426,7 +426,7 @@ abstract class AbstractContent implements Content {
 	 * @param WikiPage $page
 	 * @param ParserOutput|null $parserOutput
 	 *
-	 * @return LinksDeletionUpdate[]
+	 * @return DeferrableUpdate[]
 	 *
 	 * @see Content::getDeletionUpdates
 	 */
@@ -492,7 +492,7 @@ abstract class AbstractContent implements Content {
 	 *
 	 * @param Title $title Context title for parsing
 	 * @param int|null $revId Revision ID (for {{REVISIONID}})
-	 * @param ParserOptions|null $options Parser options
+	 * @param ParserOptions|null $options
 	 * @param bool $generateHtml Whether or not to generate HTML
 	 *
 	 * @return ParserOutput Containing information derived from this content.
@@ -501,10 +501,11 @@ abstract class AbstractContent implements Content {
 		ParserOptions $options = null, $generateHtml = true
 	) {
 		if ( $options === null ) {
-			$options = $this->getContentHandler()->makeParserOptions( 'canonical' );
+			$options = ParserOptions::newCanonical( 'canonical' );
 		}
 
 		$po = new ParserOutput();
+		$options->registerWatcher( [ $po, 'recordOption' ] );
 
 		if ( Hooks::run( 'ContentGetParserOutput',
 			[ $this, $title, $revId, $options, $generateHtml, &$po ] )
@@ -518,6 +519,7 @@ abstract class AbstractContent implements Content {
 		}
 
 		Hooks::run( 'ContentAlterParserOutput', [ $this, $title, $po ] );
+		$options->registerWatcher( null );
 
 		return $po;
 	}
@@ -536,7 +538,7 @@ abstract class AbstractContent implements Content {
 	 *
 	 * @param Title $title Context title for parsing
 	 * @param int|null $revId Revision ID (for {{REVISIONID}})
-	 * @param ParserOptions $options Parser options
+	 * @param ParserOptions $options
 	 * @param bool $generateHtml Whether or not to generate HTML
 	 * @param ParserOutput &$output The output object to fill (reference).
 	 *
