@@ -454,6 +454,7 @@ abstract class Installer {
 
 		$this->parserTitle = Title::newFromText( 'Installer' );
 		$this->parserOptions = new ParserOptions( $wgUser ); // language will be wrong :(
+		$this->parserOptions->setTidy( true );
 		// Don't try to access DB before user language is initialised
 		$this->setParserLanguage( Language::factory( 'en' ) );
 	}
@@ -531,7 +532,7 @@ abstract class Installer {
 	 * Installer variables are typically prefixed by an underscore.
 	 *
 	 * @param string $name
-	 * @param mixed $default
+	 * @param mixed|null $default
 	 *
 	 * @return mixed
 	 */
@@ -820,7 +821,7 @@ abstract class Installer {
 		// with utf8 support, but not unicode property support.
 		// check that \p{Zs} (space separators) matches
 		// U+3000 (Ideographic space)
-		$regexprop = preg_replace( '/\p{Zs}/u', '', "-\xE3\x80\x80-" );
+		$regexprop = preg_replace( '/\p{Zs}/u', '', "-\u{3000}-" );
 		Wikimedia\restoreWarnings();
 		if ( $regexd != '--' || $regexprop != '--' ) {
 			$this->showError( 'config-pcre-no-utf8' );
@@ -1116,29 +1117,6 @@ abstract class Installer {
 		return true;
 	}
 
-	/**
-	 * Convert a hex string representing a Unicode code point to that code point.
-	 * @param string $c
-	 * @return string|false
-	 */
-	protected function unicodeChar( $c ) {
-		$c = hexdec( $c );
-		if ( $c <= 0x7F ) {
-			return chr( $c );
-		} elseif ( $c <= 0x7FF ) {
-			return chr( 0xC0 | $c >> 6 ) . chr( 0x80 | $c & 0x3F );
-		} elseif ( $c <= 0xFFFF ) {
-			return chr( 0xE0 | $c >> 12 ) . chr( 0x80 | $c >> 6 & 0x3F ) .
-				chr( 0x80 | $c & 0x3F );
-		} elseif ( $c <= 0x10FFFF ) {
-			return chr( 0xF0 | $c >> 18 ) . chr( 0x80 | $c >> 12 & 0x3F ) .
-				chr( 0x80 | $c >> 6 & 0x3F ) .
-				chr( 0x80 | $c & 0x3F );
-		} else {
-			return false;
-		}
-	}
-
 	/**
 	 * Check the libicu version
 	 */
@@ -1146,12 +1124,12 @@ abstract class Installer {
 		/**
 		 * This needs to be updated something that the latest libicu
 		 * will properly normalize.  This normalization was found at
-		 * http://www.unicode.org/versions/Unicode5.2.0/#Character_Additions
+		 * https://www.unicode.org/versions/Unicode5.2.0/#Character_Additions
 		 * Note that we use the hex representation to create the code
 		 * points in order to avoid any Unicode-destroying during transit.
 		 */
-		$not_normal_c = $this->unicodeChar( "FA6C" );
-		$normal_c = $this->unicodeChar( "242EE" );
+		$not_normal_c = "\u{FA6C}";
+		$normal_c = "\u{242EE}";
 
 		$useNormalizer = 'php';
 		$needsUpdate = false;
@@ -1287,15 +1265,33 @@ abstract class Installer {
 	}
 
 	/**
-	 * Finds extensions that follow the format /$directory/Name/Name.php,
-	 * and returns an array containing the value for 'Name' for each found extension.
+	 * Find extensions or skins in a subdirectory of $IP.
+	 * Returns an array containing the value for 'Name' for each found extension.
 	 *
-	 * Reasonable values for $directory include 'extensions' (the default) and 'skins'.
-	 *
-	 * @param string $directory Directory to search in
+	 * @param string $directory Directory to search in, relative to $IP, must be either "extensions"
+	 *     or "skins"
 	 * @return array [ $extName => [ 'screenshots' => [ '...' ] ]
 	 */
 	public function findExtensions( $directory = 'extensions' ) {
+		switch ( $directory ) {
+			case 'extensions':
+				return $this->findExtensionsByType( 'extension', 'extensions' );
+			case 'skins':
+				return $this->findExtensionsByType( 'skin', 'skins' );
+			default:
+				throw new InvalidArgumentException( "Invalid extension type" );
+		}
+	}
+
+	/**
+	 * Find extensions or skins, and return an array containing the value for 'Name' for each found
+	 * extension.
+	 *
+	 * @param string $type Either "extension" or "skin"
+	 * @param string $directory Directory to search in, relative to $IP
+	 * @return array [ $extName => [ 'screenshots' => [ '...' ] ]
+	 */
+	protected function findExtensionsByType( $type = 'extension', $directory = 'extensions' ) {
 		if ( $this->getVar( 'IP' ) === null ) {
 			return [];
 		}
@@ -1305,40 +1301,15 @@ abstract class Installer {
 			return [];
 		}
 
-		// extensions -> extension.json, skins -> skin.json
-		$jsonFile = substr( $directory, 0, strlen( $directory ) - 1 ) . '.json';
-
 		$dh = opendir( $extDir );
 		$exts = [];
 		while ( ( $file = readdir( $dh ) ) !== false ) {
 			if ( !is_dir( "$extDir/$file" ) ) {
 				continue;
 			}
-			$fullJsonFile = "$extDir/$file/$jsonFile";
-			$isJson = file_exists( $fullJsonFile );
-			$isPhp = false;
-			if ( !$isJson ) {
-				// Only fallback to PHP file if JSON doesn't exist
-				$fullPhpFile = "$extDir/$file/$file.php";
-				$isPhp = file_exists( $fullPhpFile );
-			}
-			if ( $isJson || $isPhp ) {
-				// Extension exists. Now see if there are screenshots
-				$exts[$file] = [];
-				if ( is_dir( "$extDir/$file/screenshots" ) ) {
-					$paths = glob( "$extDir/$file/screenshots/*.png" );
-					foreach ( $paths as $path ) {
-						$exts[$file]['screenshots'][] = str_replace( $extDir, "../$directory", $path );
-					}
-
-				}
-			}
-			if ( $isJson ) {
-				$info = $this->readExtension( $fullJsonFile );
-				if ( $info === false ) {
-					continue;
-				}
-				$exts[$file] += $info;
+			$status = $this->getExtensionInfo( $type, $directory, $file );
+			if ( $status->isOK() ) {
+				$exts[$file] = $status->value;
 			}
 		}
 		closedir( $dh );
@@ -1347,12 +1318,65 @@ abstract class Installer {
 		return $exts;
 	}
 
+	/**
+	 * @param string $type Either "extension" or "skin"
+	 * @param string $parentRelPath The parent directory relative to $IP
+	 * @param string $name The extension or skin name
+	 * @return Status An object containing an error list. If there were no errors, an associative
+	 *     array of information about the extension can be found in $status->value.
+	 */
+	protected function getExtensionInfo( $type, $parentRelPath, $name ) {
+		if ( $this->getVar( 'IP' ) === null ) {
+			throw new Exception( 'Cannot find extensions since the IP variable is not yet set' );
+		}
+		if ( $type !== 'extension' && $type !== 'skin' ) {
+			throw new InvalidArgumentException( "Invalid extension type" );
+		}
+		$absDir = $this->getVar( 'IP' ) . "/$parentRelPath/$name";
+		$relDir = "../$parentRelPath/$name";
+		if ( !is_dir( $absDir ) ) {
+			return Status::newFatal( 'config-extension-not-found', $name );
+		}
+		$jsonFile = $type . '.json';
+		$fullJsonFile = "$absDir/$jsonFile";
+		$isJson = file_exists( $fullJsonFile );
+		$isPhp = false;
+		if ( !$isJson ) {
+			// Only fallback to PHP file if JSON doesn't exist
+			$fullPhpFile = "$absDir/$name.php";
+			$isPhp = file_exists( $fullPhpFile );
+		}
+		if ( !$isJson && !$isPhp ) {
+			return Status::newFatal( 'config-extension-not-found', $name );
+		}
+
+		// Extension exists. Now see if there are screenshots
+		$info = [];
+		if ( is_dir( "$absDir/screenshots" ) ) {
+			$paths = glob( "$absDir/screenshots/*.png" );
+			foreach ( $paths as $path ) {
+				$info['screenshots'][] = str_replace( $absDir, $relDir, $path );
+			}
+		}
+
+		if ( $isJson ) {
+			$jsonStatus = $this->readExtension( $fullJsonFile );
+			if ( !$jsonStatus->isOK() ) {
+				return $jsonStatus;
+			}
+			$info += $jsonStatus->value;
+		}
+
+		return Status::newGood( $info );
+	}
+
 	/**
 	 * @param string $fullJsonFile
 	 * @param array $extDeps
 	 * @param array $skinDeps
 	 *
-	 * @return array|bool False if this extension can't be loaded
+	 * @return Status On success, an array of extension information is in $status->value. On
+	 *    failure, the Status object will have an error list.
 	 */
 	private function readExtension( $fullJsonFile, $extDeps = [], $skinDeps = [] ) {
 		$load = [
@@ -1363,7 +1387,7 @@ abstract class Installer {
 			foreach ( $extDeps as $dep ) {
 				$fname = "$extDir/$dep/extension.json";
 				if ( !file_exists( $fname ) ) {
-					return false;
+					return Status::newFatal( 'config-extension-not-found', $dep );
 				}
 				$load[$fname] = 1;
 			}
@@ -1373,7 +1397,7 @@ abstract class Installer {
 			foreach ( $skinDeps as $dep ) {
 				$fname = "$skinDir/$dep/skin.json";
 				if ( !file_exists( $fname ) ) {
-					return false;
+					return Status::newFatal( 'config-extension-not-found', $dep );
 				}
 				$load[$fname] = 1;
 			}
@@ -1387,7 +1411,8 @@ abstract class Installer {
 			) {
 				// If something is incompatible with a dependency, we have no real
 				// option besides skipping it
-				return false;
+				return Status::newFatal( 'config-extension-dependency',
+					basename( dirname( $fullJsonFile ) ), $e->getMessage() );
 			} elseif ( $e->missingExtensions || $e->missingSkins ) {
 				// There's an extension missing in the dependency tree,
 				// so add those to the dependency list and try again
@@ -1398,7 +1423,8 @@ abstract class Installer {
 				);
 			}
 			// Some other kind of dependency error?
-			return false;
+			return Status::newFatal( 'config-extension-dependency',
+				basename( dirname( $fullJsonFile ) ), $e->getMessage() );
 		}
 		$ret = [];
 		// The order of credits will be the order of $load,
@@ -1420,7 +1446,7 @@ abstract class Installer {
 		}
 		$ret['type'] = $credits['type'];
 
-		return $ret;
+		return Status::newGood( $ret );
 	}
 
 	/**
@@ -1621,28 +1647,16 @@ abstract class Installer {
 	protected function doGenerateKeys( $keys ) {
 		$status = Status::newGood();
 
-		$strong = true;
 		foreach ( $keys as $name => $length ) {
-			$secretKey = MWCryptRand::generateHex( $length, true );
-			if ( !MWCryptRand::wasStrong() ) {
-				$strong = false;
-			}
-
+			$secretKey = MWCryptRand::generateHex( $length );
 			$this->setVar( $name, $secretKey );
 		}
 
-		if ( !$strong ) {
-			$names = array_keys( $keys );
-			$names = preg_replace( '/^(.*)$/', '\$$1', $names );
-			global $wgLang;
-			$status->warning( 'config-insecure-keys', $wgLang->listToText( $names ), count( $names ) );
-		}
-
 		return $status;
 	}
 
 	/**
-	 * Create the first user account, grant it sysop and bureaucrat rights
+	 * Create the first user account, grant it sysop, bureaucrat and interface-admin rights
 	 *
 	 * @return Status
 	 */
@@ -1666,6 +1680,7 @@ abstract class Installer {
 
 			$user->addGroup( 'sysop' );
 			$user->addGroup( 'bureaucrat' );
+			$user->addGroup( 'interface-admin' );
 			if ( $this->getVar( '_AdminEmail' ) ) {
 				$user->setEmail( $this->getVar( '_AdminEmail' ) );
 			}
@@ -1761,13 +1776,10 @@ abstract class Installer {
 		$GLOBALS['wgLanguageConverterCacheType'] = CACHE_NONE;
 		// Debug-friendly
 		$GLOBALS['wgShowExceptionDetails'] = true;
+		$GLOBALS['wgShowHostnames'] = true;
 		// Don't break forms
 		$GLOBALS['wgExternalLinkTarget'] = '_blank';
 
-		// Extended debugging
-		$GLOBALS['wgShowSQLErrors'] = true;
-		$GLOBALS['wgShowDBErrorBacktrace'] = true;
-
 		// Allow multiple ob_flush() calls
 		$GLOBALS['wgDisableOutputCompression'] = true;
 
