@@ -18,7 +18,6 @@
  * http://www.gnu.org/copyleft/gpl.html
  *
  * @file
- * @ingroup JobQueue
  */
 
 /**
@@ -58,6 +57,26 @@ interface IJobSpecification {
 	 */
 	public function getDeduplicationInfo();
 
+	/**
+	 * @see JobQueue::deduplicateRootJob()
+	 * @return array
+	 * @since 1.26
+	 */
+	public function getRootJobParams();
+
+	/**
+	 * @see JobQueue::deduplicateRootJob()
+	 * @return bool
+	 * @since 1.22
+	 */
+	public function hasRootJobParams();
+
+	/**
+	 * @see JobQueue::deduplicateRootJob()
+	 * @return bool Whether this is job is a root job
+	 */
+	public function isRootJob();
+
 	/**
 	 * @return Title Descriptive title (this can simply be informative)
 	 */
@@ -68,7 +87,7 @@ interface IJobSpecification {
  * Job queue task description base code
  *
  * Example usage:
- * <code>
+ * @code
  * $job = new JobSpecification(
  *		'null',
  *		array( 'lives' => 1, 'usleep' => 100, 'pi' => 3.141569 ),
@@ -76,7 +95,7 @@ interface IJobSpecification {
  *		Title::makeTitle( NS_SPECIAL, 'nullity' )
  * );
  * JobQueueGroup::singleton()->push( $job )
- * </code>
+ * @endcode
  *
  * @ingroup JobQueue
  * @since 1.23
@@ -97,18 +116,18 @@ class JobSpecification implements IJobSpecification {
 	/**
 	 * @param string $type
 	 * @param array $params Map of key/values
-	 * @param array $opts Map of key/values
-	 * @param Title $title Optional descriptive title
+	 * @param array $opts Map of key/values; includes 'removeDuplicates'
+	 * @param Title|null $title Optional descriptive title
 	 */
 	public function __construct(
-		$type, array $params, array $opts = array(), Title $title = null
+		$type, array $params, array $opts = [], Title $title = null
 	) {
 		$this->validateParams( $params );
 		$this->validateParams( $opts );
 
 		$this->type = $type;
 		$this->params = $params;
-		$this->title = $title ?: Title::newMainPage();
+		$this->title = $title ?: Title::makeTitle( NS_SPECIAL, 'Badtitle/' . static::class );
 		$this->opts = $opts;
 	}
 
@@ -125,58 +144,35 @@ class JobSpecification implements IJobSpecification {
 		}
 	}
 
-	/**
-	 * @return string
-	 */
 	public function getType() {
 		return $this->type;
 	}
 
-	/**
-	 * @return Title
-	 */
 	public function getTitle() {
 		return $this->title;
 	}
 
-	/**
-	 * @return array
-	 */
 	public function getParams() {
 		return $this->params;
 	}
 
-	/**
-	 * @return int|null UNIX timestamp to delay running this job until, otherwise null
-	 */
 	public function getReleaseTimestamp() {
 		return isset( $this->params['jobReleaseTimestamp'] )
 			? wfTimestampOrNull( TS_UNIX, $this->params['jobReleaseTimestamp'] )
 			: null;
 	}
 
-	/**
-	 * @return bool Whether only one of each identical set of jobs should be run
-	 */
 	public function ignoreDuplicates() {
 		return !empty( $this->opts['removeDuplicates'] );
 	}
 
-	/**
-	 * Subclasses may need to override this to make duplication detection work.
-	 * The resulting map conveys everything that makes the job unique. This is
-	 * only checked if ignoreDuplicates() returns true, meaning that duplicate
-	 * jobs are supposed to be ignored.
-	 *
-	 * @return array Map of key/values
-	 */
 	public function getDeduplicationInfo() {
-		$info = array(
+		$info = [
 			'type' => $this->getType(),
 			'namespace' => $this->getTitle()->getNamespace(),
 			'title' => $this->getTitle()->getDBkey(),
 			'params' => $this->getParams()
-		);
+		];
 		if ( is_array( $info['params'] ) ) {
 			// Identical jobs with different "root" jobs should count as duplicates
 			unset( $info['params']['rootJobSignature'] );
@@ -188,20 +184,36 @@ class JobSpecification implements IJobSpecification {
 		return $info;
 	}
 
+	public function getRootJobParams() {
+		return [
+			'rootJobSignature' => $this->params['rootJobSignature'] ?? null,
+			'rootJobTimestamp' => $this->params['rootJobTimestamp'] ?? null
+		];
+	}
+
+	public function hasRootJobParams() {
+		return isset( $this->params['rootJobSignature'] )
+			&& isset( $this->params['rootJobTimestamp'] );
+	}
+
+	public function isRootJob() {
+		return $this->hasRootJobParams() && !empty( $this->params['rootJobIsSelf'] );
+	}
+
 	/**
 	 * @return array Field/value map that can immediately be serialized
 	 * @since 1.25
 	 */
 	public function toSerializableArray() {
-		return array(
+		return [
 			'type'   => $this->type,
 			'params' => $this->params,
 			'opts'   => $this->opts,
-			'title'  => array(
+			'title'  => [
 				'ns'  => $this->title->getNamespace(),
-				'key' => $this->title->getDbKey()
-			)
-		);
+				'key' => $this->title->getDBkey()
+			]
+		];
 	}
 
 	/**
