@@ -18,8 +18,8 @@
  * http://www.gnu.org/copyleft/gpl.html
  *
  * @file
- * @author Aaron Schulz
  */
+use Psr\Log\LoggerInterface;
 
 /**
  * Class to handle tracking information about all queues using PhpRedis
@@ -33,6 +33,8 @@
 class JobQueueAggregatorRedis extends JobQueueAggregator {
 	/** @var RedisConnectionPool */
 	protected $redisPool;
+	/** @var LoggerInterface */
+	protected $logger;
 	/** @var array List of Redis server addresses */
 	protected $servers;
 
@@ -47,11 +49,10 @@ class JobQueueAggregatorRedis extends JobQueueAggregator {
 	 */
 	public function __construct( array $params ) {
 		parent::__construct( $params );
-		$this->servers = isset( $params['redisServers'] )
-			? $params['redisServers']
-			: [ $params['redisServer'] ]; // b/c
+		$this->servers = $params['redisServers'] ?? [ $params['redisServer'] ]; // b/c
 		$params['redisConfig']['serializer'] = 'none';
 		$this->redisPool = RedisConnectionPool::singleton( $params['redisConfig'] );
+		$this->logger = \MediaWiki\Logger\LoggerFactory::getInstance( 'redis' );
 	}
 
 	protected function doNotifyQueueEmpty( $wiki, $type ) {
@@ -104,7 +105,7 @@ class JobQueueAggregatorRedis extends JobQueueAggregator {
 	protected function getConnection() {
 		$conn = false;
 		foreach ( $this->servers as $server ) {
-			$conn = $this->redisPool->getConnection( $server );
+			$conn = $this->redisPool->getConnection( $server, $this->logger );
 			if ( $conn ) {
 				break;
 			}
@@ -122,7 +123,7 @@ class JobQueueAggregatorRedis extends JobQueueAggregator {
 
 	/**
 	 * @param string $name
-	 * @return string
+	 * @return string[]
 	 */
 	private function decodeQueueName( $name ) {
 		list( $type, $wiki ) = explode( '/', $name, 2 );
