@@ -88,7 +88,7 @@ class SpecialPasswordReset extends FormSpecialPage {
 		$a = [];
 		if ( isset( $resetRoutes['username'] ) && $resetRoutes['username'] ) {
 			$a['Username'] = [
-				'type' => 'text',
+				'type' => 'user',
 				'label-message' => 'passwordreset-username',
 			];
 
@@ -114,6 +114,8 @@ class SpecialPasswordReset extends FormSpecialPage {
 	public function alterForm( HTMLForm $form ) {
 		$resetRoutes = $this->getConfig()->get( 'PasswordResetRoutes' );
 
+		$form->setSubmitDestructive();
+
 		$form->addHiddenFields( $this->getRequest()->getValues( 'returnto', 'returntoquery' ) );
 
 		$i = 0;
@@ -140,8 +142,8 @@ class SpecialPasswordReset extends FormSpecialPage {
 	 * @return Status
 	 */
 	public function onSubmit( array $data ) {
-		$username = isset( $data['Username'] ) ? $data['Username'] : null;
-		$email = isset( $data['Email'] ) ? $data['Email'] : null;
+		$username = $data['Username'] ?? null;
+		$email = $data['Email'] ?? null;
 
 		$this->method = $username ? 'username' : 'email';
 		$this->result = Status::wrap(
