@@ -21,6 +21,8 @@
  * @ingroup Maintenance
  */
 
+use MediaWiki\Revision\SlotRecord;
+
 require_once __DIR__ . '/Maintenance.php';
 
 /**
@@ -31,33 +33,39 @@ require_once __DIR__ . '/Maintenance.php';
 class EditCLI extends Maintenance {
 	public function __construct() {
 		parent::__construct();
-		$this->mDescription = "Edit an article from the command line, text is from stdin";
+		$this->addDescription( 'Edit an article from the command line, text is from stdin' );
 		$this->addOption( 'user', 'Username', false, true, 'u' );
 		$this->addOption( 'summary', 'Edit summary', false, true, 's' );
+		$this->addOption( 'remove', 'Remove a slot (requires --slot).', false, false );
 		$this->addOption( 'minor', 'Minor edit', false, false, 'm' );
 		$this->addOption( 'bot', 'Bot edit', false, false, 'b' );
 		$this->addOption( 'autosummary', 'Enable autosummary', false, false, 'a' );
 		$this->addOption( 'no-rc', 'Do not show the change in recent changes', false, false, 'r' );
 		$this->addOption( 'nocreate', 'Don\'t create new pages', false, false );
 		$this->addOption( 'createonly', 'Only create new pages', false, false );
+		$this->addOption( 'slot', 'Slot role name', false, true );
 		$this->addArg( 'title', 'Title of article to edit' );
 	}
 
 	public function execute() {
 		global $wgUser;
 
-		$userName = $this->getOption( 'user', 'Maintenance script' );
+		$userName = $this->getOption( 'user', false );
 		$summary = $this->getOption( 'summary', '' );
+		$remove = $this->hasOption( 'remove' );
 		$minor = $this->hasOption( 'minor' );
 		$bot = $this->hasOption( 'bot' );
 		$autoSummary = $this->hasOption( 'autosummary' );
 		$noRC = $this->hasOption( 'no-rc' );
+		$slot = $this->getOption( 'slot', SlotRecord::MAIN );
 
-		$wgUser = User::newFromName( $userName );
-		$context = RequestContext::getMain();
-		$context->setUser( $wgUser );
+		if ( $userName === false ) {
+			$wgUser = User::newSystemUser( 'Maintenance script', [ 'steal' => true ] );
+		} else {
+			$wgUser = User::newFromName( $userName );
+		}
 		if ( !$wgUser ) {
-			$this->error( "Invalid username", true );
+			$this->fatalError( "Invalid username" );
 		}
 		if ( $wgUser->isAnon() ) {
 			$wgUser->addToDatabase();
@@ -65,29 +73,47 @@ class EditCLI extends Maintenance {
 
 		$title = Title::newFromText( $this->getArg() );
 		if ( !$title ) {
-			$this->error( "Invalid title", true );
+			$this->fatalError( "Invalid title" );
 		}
-		$context->setTitle( $title );
 
 		if ( $this->hasOption( 'nocreate' ) && !$title->exists() ) {
-			$this->error( "Page does not exist", true );
+			$this->fatalError( "Page does not exist" );
 		} elseif ( $this->hasOption( 'createonly' ) && $title->exists() ) {
-			$this->error( "Page already exists", true );
+			$this->fatalError( "Page already exists" );
 		}
 
 		$page = WikiPage::factory( $title );
 
-		# Read the text
-		$text = $this->getStdin( Maintenance::STDIN_ALL );
-		$content = ContentHandler::makeContent( $text, $title );
+		if ( $remove ) {
+			if ( $slot === SlotRecord::MAIN ) {
+				$this->fatalError( "Cannot remove main slot! Use --slot to specify." );
+			}
+
+			$content = false;
+		} else {
+			# Read the text
+			$text = $this->getStdin( Maintenance::STDIN_ALL );
+			$content = ContentHandler::makeContent( $text, $title );
+		}
 
 		# Do the edit
 		$this->output( "Saving... " );
-		$status = $page->doEditContent( $content, $summary,
-			( $minor ? EDIT_MINOR : 0 ) |
+		$updater = $page->newPageUpdater( $wgUser );
+
+		$flags = ( $minor ? EDIT_MINOR : 0 ) |
 			( $bot ? EDIT_FORCE_BOT : 0 ) |
 			( $autoSummary ? EDIT_AUTOSUMMARY : 0 ) |
-			( $noRC ? EDIT_SUPPRESS_RC : 0 ) );
+			( $noRC ? EDIT_SUPPRESS_RC : 0 );
+
+		if ( $content === false ) {
+			$updater->removeSlot( $slot );
+		} else {
+			$updater->setContent( $slot, $content );
+		}
+
+		$updater->saveRevision( CommentStoreComment::newUnsavedComment( $summary ), $flags );
+		$status = $updater->getStatus();
+
 		if ( $status->isOK() ) {
 			$this->output( "done\n" );
 			$exit = 0;
@@ -96,11 +122,11 @@ class EditCLI extends Maintenance {
 			$exit = 1;
 		}
 		if ( !$status->isGood() ) {
-			$this->output( $status->getWikiText() . "\n" );
+			$this->output( $status->getWikiText( false, false, 'en' ) . "\n" );
 		}
 		exit( $exit );
 	}
 }
 
-$maintClass = "EditCLI";
+$maintClass = EditCLI::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
